﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Demo
{
    public class EstopButton : Control
    {
        #region Konstruktor
        public EstopButton() : base()
        {

            SetStyle(ControlStyles.AllPaintingInWmPaint |
                      ControlStyles.OptimizedDoubleBuffer |
                      ControlStyles.ResizeRedraw |
                      ControlStyles.SupportsTransparentBackColor |
                      ControlStyles.UserPaint
                      , true);

            Size = new Size(75, 75);
            Font = new Font("Segoe UI", 8.25f, FontStyle.Bold);
        }
        #endregion

        #region Variablen / Konstanten

        private bool _mouseHover;

        private readonly Color Red = Color.FromArgb(214, 79, 46);

        #endregion

        #region Funktionen

        private void DrawCurvedText(Graphics graphics, string text, Point centre, float distanceFromCentreToBaseOfText, float radiansToTextCentre, Font font, Brush brush)
        {
            // Circumference for use later
            var circleCircumference = (float)(Math.PI * 2 * distanceFromCentreToBaseOfText);

            // Get the width of each character
            var characterWidths = GetCharacterWidths(graphics, text, font).ToArray();

            // The overall height of the string
            var characterHeight = graphics.MeasureString(text, font).Height;

            var textLength = characterWidths.Sum();

            // The string length above is the arc length we'll use for rendering the string. Work out the starting angle required to 
            // centre the text across the radiansToTextCentre.
            float fractionOfCircumference = textLength / circleCircumference;

            float currentCharacterRadians = radiansToTextCentre - (float)(Math.PI * fractionOfCircumference);

            for (int characterIndex = 0; characterIndex < text.Length; characterIndex++)
            {
                char @char = text[characterIndex];

                // Polar to cartesian
                float x = (float)(distanceFromCentreToBaseOfText * Math.Sin(currentCharacterRadians));
                float y = -(float)(distanceFromCentreToBaseOfText * Math.Cos(currentCharacterRadians));

                using (GraphicsPath characterPath = new GraphicsPath())
                {
                    characterPath.AddString(@char.ToString(), font.FontFamily, (int)font.Style, font.Size, Point.Empty,
                                            StringFormat.GenericTypographic);

                    var pathBounds = characterPath.GetBounds();

                    // Transformation matrix to move the character to the correct location. 
                    // Note that all actions on the Matrix class are prepended, so we apply them in reverse.
                    var transform = new Matrix();

                    // Translate to the final position
                    transform.Translate(centre.X + x, centre.Y + y);

                    // Rotate the character
                    var rotationAngleDegrees = currentCharacterRadians * 180f / (float)Math.PI/*- 180F*/;
                    transform.Rotate(rotationAngleDegrees);

                    // Translate the character so the centre of its base is over the origin
                    transform.Translate(-pathBounds.Width / 2F, -characterHeight);

                    characterPath.Transform(transform);

                    // Draw the character
                    graphics.FillPath(brush, characterPath);
                }

                if (characterIndex != text.Length - 1)
                {
                    // Move "currentCharacterRadians" on to the next character
                    var distanceToNextChar = (characterWidths[characterIndex] + characterWidths[characterIndex + 1]) / 2F;
                    float charFractionOfCircumference = distanceToNextChar / circleCircumference;
                    currentCharacterRadians += charFractionOfCircumference * (float)(2F * Math.PI);
                }
            }
        }

        private IEnumerable<float> GetCharacterWidths(Graphics graphics, string text, Font font)
        {
            // The length of a space. Necessary because a space measured using StringFormat.GenericTypographic has no width.
            // We can't use StringFormat.GenericDefault for the characters themselves, as it adds unwanted spacing.
            var spaceLength = graphics.MeasureString(" ", font, Point.Empty, StringFormat.GenericDefault).Width;

            return text.Select(c => c == ' ' ? spaceLength : graphics.MeasureString(c.ToString(), font, Point.Empty, StringFormat.GenericTypographic).Width);
        }

        #endregion

        #region Callbacks

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            #region gelber Kreis
            // Leinwand zum Zeichnen definieren
            var squareClientRectYellow = ClientRectangle;
            squareClientRectYellow.Width = squareClientRectYellow.Width > squareClientRectYellow.Height ? squareClientRectYellow.Height - 2 : squareClientRectYellow.Width - 2;
            squareClientRectYellow.Height = squareClientRectYellow.Height > squareClientRectYellow.Width ? squareClientRectYellow.Width - 2 : squareClientRectYellow.Height - 2;

            // Quadrat zentrieren
            squareClientRectYellow.X = (int)(Width / 2f - squareClientRectYellow.Width / 2f);
            squareClientRectYellow.Y = (int)(Height / 2f - squareClientRectYellow.Height / 2f);

            // Gelben Kreis zeichnen
            e.Graphics.FillEllipse(new SolidBrush(Color.FromArgb(255, 192, 0)), squareClientRectYellow);
            #endregion

            #region roter Kreis
            // Leinwand zum Zeichnen definieren
            int percentage = 65;
            var squareClientRectRed = new Rectangle(0, 0, (int)(squareClientRectYellow.Width / 100f * percentage), (int)(squareClientRectYellow.Height / 100f * percentage));

            // Quadrat zentrieren
            squareClientRectRed.X = (int)((Width / 2f) - (squareClientRectRed.Width / 2f));
            squareClientRectRed.Y = (int)((Height / 2f) - (squareClientRectRed.Height / 2f));

            // Roten Kreis zeichnen
            e.Graphics.DrawEllipse(new Pen(ControlPaint.Dark(Red), 2), squareClientRectRed);
            e.Graphics.FillEllipse(new SolidBrush(!_mouseHover ? Red : ControlPaint.Light(Red)), squareClientRectRed);
            #endregion

            var textSize = e.Graphics.MeasureString(Text, Font);
            var textDist = (squareClientRectYellow.Width / 2 + squareClientRectRed.Width / 2) / 2 - textSize.Height / 1.5f;

            DrawCurvedText(e.Graphics, Text, new Point(Width / 2, Height / 2), textDist, 0, Font, new SolidBrush(Color.Black));

        }

        protected override void OnTextChanged(EventArgs e)
        {
            base.OnTextChanged(e);
            Invalidate();
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);
            _mouseHover = true;
            Invalidate();
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            _mouseHover = false;
            Invalidate();
        }

        #endregion

    }
}
